@@ -20,6 +20,9 @@ import time
 import pickle
 
 from setzer.document.document import Document
+from setzer.document.document_latex import DocumentLaTeX
+from setzer.document.document_bibtex import DocumentBibTeX
+from setzer.document.document_other import DocumentOther
 from setzer.helpers.observable import Observable
 import setzer.workspace.workspace_presenter as workspace_presenter
 import setzer.workspace.workspace_controller as workspace_controller
@@ -67,7 +70,8 @@ class Workspace(Observable):
 
         self.sidebar = sidebar.Sidebar(self)
         self.welcome_screen = welcome_screen.WelcomeScreen()
-        self.show_sidebar = self.settings.get_value('window_state', 'show_sidebar')
+        self.show_symbols = self.settings.get_value('window_state', 'show_symbols')
+        self.show_document_structure = self.settings.get_value('window_state', 'show_document_structure')
         self.sidebar_position = self.settings.get_value('window_state', 'sidebar_paned_position')
         self.show_help = self.settings.get_value('window_state', 'show_help')
         self.show_preview = self.settings.get_value('window_state', 'show_preview')
@@ -138,28 +142,21 @@ class Workspace(Observable):
         self.add_change_code('document_removed', document)
 
     def create_latex_document(self):
-        document = Document('latex')
-        document.add_latex_only_modules()
-        return document
+        return DocumentLaTeX()
 
     def create_bibtex_document(self):
-        return Document('bibtex')
+        return DocumentBibTeX()
 
-    def create_latex_cls_document(self):
-        return Document('latex_cls')
-
-    def create_latex_sty_document(self):
-        return Document('latex_sty')
+    def create_other_document(self):
+        return DocumentOther()
 
     def create_document_from_filename(self, filename):
         if filename[-4:] == '.tex':
             document = self.create_latex_document()
         elif filename[-4:] == '.bib':
             document = self.create_bibtex_document()
-        elif filename[-4:] == '.cls':
-            document = self.create_latex_cls_document()
-        elif filename[-4:] == '.sty':
-            document = self.create_latex_sty_document()
+        elif filename[-4:] in ['.cls', '.sty']:
+            document = self.create_other_document()
         else:
             return None
         document.set_filename(filename)
@@ -200,13 +197,9 @@ class Workspace(Observable):
             self.set_build_log()
 
     def set_build_log(self):
-        if self.get_active_document() != None:
-            if self.root_document != None:
-                document = self.root_document
-            else:
-                document = self.active_document
-            if document.is_latex_document():
-                self.build_log.set_document(document)
+        document = self.get_root_or_active_latex_document()
+        if document != None:
+            self.build_log.set_document(document)
 
     def get_last_active_document(self):
         for document in sorted(self.open_documents, key=lambda val: -val.last_activated):
@@ -393,14 +386,33 @@ class Workspace(Observable):
     def get_root_document(self):
         return self.root_document
 
+    def get_root_or_active_latex_document(self):
+        if self.get_active_document() == None:
+            return None
+        else:
+            if self.root_document != None:
+                return self.root_document
+            elif self.active_document.is_latex_document():
+                return self.active_document
+            else:
+                return None
+
     def update_preview_visibility(self, document):
         if document != None and document.is_latex_document():
             if document == self.root_document:
-                document.preview.set_is_visible(True)
+                document.preview.page_renderer.activate()
             elif document == self.active_document and self.root_document == None:
-                document.preview.set_is_visible(True)
+                document.preview.page_renderer.activate()
             else:
-                document.preview.set_is_visible(False)
+                document.preview.page_renderer.deactivate()
+
+    def update_sync_document(self):
+        document = self.get_root_or_active_latex_document()
+        if document != None:
+            self.set_sync_document(document)
+            self.sync_document.disconnect('is_root_changed', self.on_is_root_changed)
+            self.sync_document.build_system.disconnect('can_sync_changed', self.on_can_sync_changed)
+            self.sync_document = None
 
     def update_sync_document(self):
         if self.root_document != None:
@@ -414,15 +426,17 @@ class Workspace(Observable):
             self.sync_document = None
 
     def set_sync_document(self, document):
-        if document != self.sync_document:
-            if self.sync_document != None:
-                self.sync_document.disconnect('is_root_changed', self.on_is_root_changed)
-                if self.sync_document.is_latex_document():
-                    self.sync_document.build_system.disconnect('can_sync_changed', self.on_can_sync_changed)
-            self.sync_document = document
-            self.sync_document.connect('is_root_changed', self.on_is_root_changed)
+        if document == None: return
+        if document == self.sync_document: return
+
+        if self.sync_document != None:
+            self.sync_document.disconnect('is_root_changed', self.on_is_root_changed)
             if self.sync_document.is_latex_document():
-                self.sync_document.build_system.connect('can_sync_changed', self.on_can_sync_changed)
+                self.sync_document.build_system.disconnect('can_sync_changed', self.on_can_sync_changed)
+        self.sync_document = document
+        self.sync_document.connect('is_root_changed', self.on_is_root_changed)
+        if self.sync_document.is_latex_document():
+            self.sync_document.build_system.connect('can_sync_changed', self.on_can_sync_changed)
 
     def on_can_sync_changed(self, build_system, can_sync):
         self.set_can_sync()
@@ -443,15 +457,15 @@ class Workspace(Observable):
         if active_document == None: return
         if not self.sync_document.is_latex_document(): return
 
-        if self.root_document != None:
-            self.root_document.build_system.forward_sync(active_document)
-        else:
-            active_document.build_system.forward_sync(active_document)
+        document = self.get_root_or_active_latex_document()
+        if document != None:
+            document.build_system.forward_sync(active_document)
 
-    def set_show_sidebar(self, show_sidebar):
-        if show_sidebar != self.show_sidebar:
-            self.show_sidebar = show_sidebar
-            self.add_change_code('set_show_sidebar', show_sidebar)
+    def set_show_symbols_or_document_structure(self, show_symbols, show_document_structure):
+        if show_symbols != self.show_symbols or show_document_structure != self.show_document_structure:
+            self.show_symbols = show_symbols
+            self.show_document_structure = show_document_structure
+            self.add_change_code('set_show_symbols_or_document_structure')
 
     def set_show_preview_or_help(self, show_preview, show_help):
         if show_preview != self.show_preview or show_help != self.show_help:
